@@ -1,9 +1,9 @@
 /*
- *  $Id: arrows.c,v 1.51 2013/09/02 15:10:09 tom Exp $
+ *  $Id: arrows.c,v 1.53 2019/07/24 23:04:14 tom Exp $
  *
  *  arrows.c -- draw arrows to indicate end-of-range for lists
  *
- *  Copyright 2000-2012,2013	Thomas E. Dickey
+ *  Copyright 2000-2018,2019	Thomas E. Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License, version 2.1
@@ -70,7 +70,6 @@ merge_colors(chtype foreground, chtype background)
 void
 dlg_draw_helpline(WINDOW *win, bool decorations)
 {
-    int cur_x, cur_y;
     int bottom;
 
     if (dialog_vars.help_line != 0
@@ -83,6 +82,8 @@ dlg_draw_helpline(WINDOW *win, bool decorations)
 	int limit = dlg_count_real_columns(dialog_vars.help_line) + 2;
 
 	if (limit < avail) {
+	    int cur_x, cur_y;
+
 	    getyx(win, cur_y, cur_x);
 	    other = decorations ? ON_LEFT : 0;
 	    (void) wmove(win, bottom, other + (avail - limit) / 2);
@@ -125,11 +126,11 @@ dlg_draw_arrows2(WINDOW *win,
     if (draw_top) {
 	(void) wmove(win, top, x);
 	if (top_arrow) {
-	    (void) wattrset(win, merge_colors(uarrow_attr, attr));
+	    dlg_attrset(win, merge_colors(uarrow_attr, attr));
 	    (void) add_acs(win, ACS_UARROW);
 	    (void) waddstr(win, "(-)");
 	} else {
-	    (void) wattrset(win, attr);
+	    dlg_attrset(win, attr);
 	    (void) whline(win, dlg_boxchar(ACS_HLINE), ON_LEFT);
 	}
     }
@@ -137,11 +138,11 @@ dlg_draw_arrows2(WINDOW *win,
 
     (void) wmove(win, bottom, x);
     if (bottom_arrow) {
-	(void) wattrset(win, merge_colors(darrow_attr, borderattr));
+	dlg_attrset(win, merge_colors(darrow_attr, borderattr));
 	(void) add_acs(win, ACS_DARROW);
 	(void) waddstr(win, "(+)");
     } else {
-	(void) wattrset(win, borderattr);
+	dlg_attrset(win, borderattr);
 	(void) whline(win, dlg_boxchar(ACS_HLINE), ON_LEFT);
     }
     mouse_mkbutton(bottom, x - 1, 6, KEY_NPAGE);
@@ -149,7 +150,7 @@ dlg_draw_arrows2(WINDOW *win,
     (void) wmove(win, cur_y, cur_x);
     wrefresh(win);
 
-    (void) wattrset(win, save);
+    dlg_attrset(win, save);
 }
 
 void
@@ -165,9 +166,6 @@ dlg_draw_scrollbar(WINDOW *win,
 		   chtype attr,
 		   chtype borderattr)
 {
-    char buffer[80];
-    int percent;
-    int len;
     int oldy, oldx;
 
     chtype save = dlg_get_attrs(win);
@@ -178,6 +176,10 @@ dlg_draw_scrollbar(WINDOW *win,
 
     dlg_draw_helpline(win, TRUE);
     if (bottom_arrow || top_arrow || dialog_state.use_scrollbar) {
+	char buffer[80];
+	int percent;
+	int len;
+
 	percent = (!total_data
 		   ? 100
 		   : (int) ((next_data * 100)
@@ -188,12 +190,12 @@ dlg_draw_scrollbar(WINDOW *win,
 	else if (percent > 100)
 	    percent = 100;
 
-	(void) wattrset(win, position_indicator_attr);
+	dlg_attrset(win, position_indicator_attr);
 	(void) sprintf(buffer, "%d%%", percent);
 	(void) wmove(win, bottom, right - 7);
 	(void) waddstr(win, buffer);
 	if ((len = dlg_count_columns(buffer)) < 4) {
-	    (void) wattrset(win, border_attr);
+	    dlg_attrset(win, border_attr);
 	    whline(win, dlg_boxchar(ACS_HLINE), 4 - len);
 	}
     }
@@ -209,7 +211,6 @@ dlg_draw_scrollbar(WINDOW *win,
 	    int all_diff = (int) (total_data + 1);
 	    int bar_diff = (int) (next_data + 1 - this_data);
 	    int bar_high;
-	    int bar_y;
 
 	    bar_high = ORDSIZE(bar_diff);
 	    if (bar_high <= 0)
@@ -217,10 +218,11 @@ dlg_draw_scrollbar(WINDOW *win,
 
 	    if (bar_high < all_high) {
 		int bar_last = BARSIZE(next_data);
+		int bar_y;
 
 		wmove(win, top + 1, right);
 
-		(void) wattrset(win, save);
+		dlg_attrset(win, save);
 		wvline(win, ACS_VLINE | A_REVERSE, all_high);
 
 		bar_y = ORDSIZE(this_data);
@@ -232,8 +234,8 @@ dlg_draw_scrollbar(WINDOW *win,
 
 		wmove(win, top + 1 + bar_y, right);
 
-		(void) wattrset(win, position_indicator_attr);
-		wattron(win, A_REVERSE);
+		dlg_attrset(win, position_indicator_attr);
+		dlg_attron(win, A_REVERSE);
 #if defined(WACS_BLOCK) && defined(NCURSES_VERSION) && defined(USE_WIDE_CURSES)
 		wvline_set(win, WACS_BLOCK, bar_last - bar_y);
 #else
@@ -251,7 +253,7 @@ dlg_draw_scrollbar(WINDOW *win,
 		     attr,
 		     borderattr);
 
-    (void) wattrset(win, save);
+    dlg_attrset(win, save);
     wmove(win, oldy, oldx);
 }
 
