@@ -1,9 +1,9 @@
 /*
- *  $Id: dlg_keys.c,v 1.42 2017/01/31 00:26:04 tom Exp $
+ *  $Id: dlg_keys.c,v 1.54 2019/08/05 23:33:46 tom Exp $
  *
  *  dlg_keys.c -- runtime binding support for dialog
  *
- *  Copyright 2006-2016,2017 Thomas E. Dickey
+ *  Copyright 2006-2018,2019 Thomas E. Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License, version 2.1
@@ -61,10 +61,11 @@ dlg_register_window(WINDOW *win, const char *name, DLG_KEYS_BINDING * binding)
 	p->win = win;
 	p->name = name;
 	p->binding = binding;
-	if (q != 0)
+	if (q != 0) {
 	    q->link = p;
-	else
+	} else {
 	    all_bindings = p;
+	}
     }
 #if defined(HAVE_DLG_TRACE) && defined(HAVE_RC_FILE)
     /*
@@ -75,7 +76,9 @@ dlg_register_window(WINDOW *win, const char *name, DLG_KEYS_BINDING * binding)
      * than by running dialog and tracing it.
      */
     DLG_TRACE(("# dlg_register_window %s\n", name));
+    dlg_dump_keys(dialog_state.trace_output);
     dlg_dump_window_keys(dialog_state.trace_output, win);
+    DLG_TRACE(("# ...done dlg_register_window %s\n", name));
 #endif
 }
 
@@ -198,8 +201,8 @@ dlg_unregister_window(WINDOW *win)
  * Parameters:
  *	win is the window on which the wgetch() was done.
  *	curses_key is the value returned by wgetch().
- *	fkey in/out (on input, it is true if curses_key is a function key,
- *		and on output, it is true if the result is a function key).
+ *	fkey in/out (on input, it is nonzero if curses_key is a function key,
+ *		and on output, it is nonzero if the result is a function key).
  */
 int
 dlg_lookup_key(WINDOW *win, int curses_key, int *fkey)
@@ -271,12 +274,26 @@ dlg_result_key(int dialog_key, int fkey GCC_UNUSED, int *resultp)
 {
     int done = FALSE;
 
+    dlg_trace_msg("# dlg_result_key(dialog_key=%d, fkey=%d)\n", dialog_key, fkey);
+#ifdef KEY_RESIZE
+    if (dialog_state.had_resize) {
+	if (dialog_key == ERR) {
+	    dialog_key = 0;
+	} else {
+	    dialog_state.had_resize = FALSE;
+	}
+    } else if (fkey && dialog_key == KEY_RESIZE) {
+	dialog_state.had_resize = TRUE;
+    }
+#endif
 #ifdef HAVE_RC_FILE
     if (fkey) {
 	switch ((DLG_KEYS_ENUM) dialog_key) {
 	case DLGK_OK:
-	    *resultp = DLG_EXIT_OK;
-	    done = TRUE;
+	    if (!dialog_vars.nook) {
+		*resultp = DLG_EXIT_OK;
+		done = TRUE;
+	    }
 	    break;
 	case DLGK_CANCEL:
 	    if (!dialog_vars.nocancel) {
@@ -316,6 +333,70 @@ dlg_result_key(int dialog_key, int fkey GCC_UNUSED, int *resultp)
     return done;
 }
 
+/*
+ * If a key was bound to one of the button-codes in dlg_result_key(), fake
+ * a button-value and an "Enter" key to cause the calling widget to return
+ * the corresponding status.
+ *
+ * See dlg_ok_buttoncode(), which maps settings for ok/extra/help and button
+ * number into exit-code.
+ */
+int
+dlg_button_key(int exit_code, int *button, int *dialog_key, int *fkey)
+{
+    int changed = FALSE;
+    switch (exit_code) {
+    case DLG_EXIT_OK:
+	if (!dialog_vars.nook) {
+	    *button = 0;
+	    changed = TRUE;
+	}
+	break;
+    case DLG_EXIT_EXTRA:
+	if (dialog_vars.extra_button) {
+	    *button = dialog_vars.nook ? 0 : 1;
+	    changed = TRUE;
+	}
+	break;
+    case DLG_EXIT_CANCEL:
+	if (!dialog_vars.nocancel) {
+	    *button = dialog_vars.nook ? 1 : 2;
+	    changed = TRUE;
+	}
+	break;
+    case DLG_EXIT_HELP:
+	if (dialog_vars.help_button) {
+	    int cancel = dialog_vars.nocancel ? 0 : 1;
+	    int extra = dialog_vars.extra_button ? 1 : 0;
+	    int okay = dialog_vars.nook ? 0 : 1;
+	    *button = okay + extra + cancel;
+	    changed = TRUE;
+	}
+	break;
+    }
+    if (changed) {
+	dlg_trace_msg("# dlg_button_key(%d:%s) button %d\n",
+		      exit_code, dlg_exitcode2s(exit_code), *button);
+	*dialog_key = *fkey = DLGK_ENTER;
+    }
+    return changed;
+}
+
+int
+dlg_ok_button_key(int exit_code, int *button, int *dialog_key, int *fkey)
+{
+    int result;
+    DIALOG_VARS save;
+
+    dlg_save_vars(&save);
+    dialog_vars.nocancel = TRUE;
+
+    result = dlg_button_key(exit_code, button, dialog_key, fkey);
+
+    dlg_restore_vars(&save);
+    return result;
+}
+
 #ifdef HAVE_RC_FILE
 typedef struct {
     const char *name;
@@ -560,13 +641,13 @@ make_binding(char *widget, int curses_key, int is_function, int dialog_key)
     LIST_BINDINGS *entry = 0;
     DLG_KEYS_BINDING *data = 0;
     char *name;
-    LIST_BINDINGS *p, *q;
     DLG_KEYS_BINDING *result = find_binding(widget, curses_key);
 
     if (result == 0
 	&& (entry = dlg_calloc(LIST_BINDINGS, 1)) != 0
 	&& (data = dlg_calloc(DLG_KEYS_BINDING, 2)) != 0
 	&& (name = dlg_strclone(widget)) != 0) {
+	LIST_BINDINGS *p, *q;
 
 	entry->name = name;
 	entry->binding = data;
@@ -603,7 +684,6 @@ make_binding(char *widget, int curses_key, int is_function, int dialog_key)
 static int
 decode_escaped(char **string)
 {
-    unsigned n;
     int result = 0;
 
     if (IsOctal(**string)) {
@@ -614,6 +694,8 @@ decode_escaped(char **string)
 	    result = (result << 3) | (ch - '0');
 	}
     } else {
+	unsigned n;
+
 	for (n = 0; n < TableSize(escaped_letters); ++n) {
 	    if (**string == escaped_letters[n].letter) {
 		*string += 1;
@@ -660,13 +742,8 @@ int
 dlg_parse_bindkey(char *params)
 {
     char *p = skip_white(params);
-    char *q;
-    bool escaped = FALSE;
-    int modified = 0;
     int result = FALSE;
-    unsigned xx;
     char *widget;
-    int is_function = FALSE;
     int curses_key;
     int dialog_key;
 
@@ -676,6 +753,12 @@ dlg_parse_bindkey(char *params)
 
     p = skip_black(p);
     if (p != widget && *p != '\0') {
+	char *q;
+	unsigned xx;
+	bool escaped = FALSE;
+	int modified = 0;
+	int is_function = FALSE;
+
 	*p++ = '\0';
 	p = skip_white(p);
 	q = p;
@@ -712,7 +795,7 @@ dlg_parse_bindkey(char *params)
 		char fprefix[2];
 		char check[2];
 		int keynumber;
-		if (sscanf(q, "%[Ff]%d%c", fprefix, &keynumber, check) == 2) {
+		if (sscanf(q, "%1[Ff]%d%c", fprefix, &keynumber, check) == 2) {
 		    curses_key = KEY_F(keynumber);
 		    is_function = TRUE;
 		} else {
@@ -807,12 +890,28 @@ dump_dialog_key(FILE *fp, int dialog_key)
 }
 
 static void
-dump_one_binding(FILE *fp, const char *widget, DLG_KEYS_BINDING * binding)
+dump_one_binding(FILE *fp,
+		 WINDOW *win,
+		 const char *widget,
+		 DLG_KEYS_BINDING * binding)
 {
+    int actual;
+    int fkey = (binding->curses_key > 255);
+
     fprintf(fp, "bindkey %s ", widget);
     dump_curses_key(fp, binding->curses_key);
     fputc(' ', fp);
     dump_dialog_key(fp, binding->dialog_key);
+    actual = dlg_lookup_key(win, binding->curses_key, &fkey);
+#ifdef KEY_MOUSE
+    if (is_DLGK_MOUSE(binding->curses_key) && is_DLGK_MOUSE(actual)) {
+	;			/* EMPTY */
+    } else
+#endif
+    if (actual != binding->dialog_key) {
+	fprintf(fp, "\t# overridden by ");
+	dump_dialog_key(fp, actual);
+    }
     fputc('\n', fp);
 }
 
@@ -832,12 +931,13 @@ dlg_dump_window_keys(FILE *fp, WINDOW *win)
 	for (p = all_bindings; p != 0; p = p->link) {
 	    if (p->win == win) {
 		if (dlg_strcmp(last, p->name)) {
-		    fprintf(fp, "\n# key bindings for %s widgets\n",
-			    !strcmp(p->name, WILDNAME) ? "all" : p->name);
+		    fprintf(fp, "# key bindings for %s widgets%s\n",
+			    !strcmp(p->name, WILDNAME) ? "all" : p->name,
+			    win == 0 ? " (user-defined)" : "");
 		    last = p->name;
 		}
 		for (q = p->binding; q->is_function_key >= 0; ++q) {
-		    dump_one_binding(fp, p->name, q);
+		    dump_one_binding(fp, win, p->name, q);
 		}
 	    }
 	}
