@@ -1,9 +1,9 @@
 /*
- *  $Id: formbox.c,v 1.80 2011/10/20 23:34:21 tom Exp $
+ *  $Id: formbox.c,v 1.97 2019/08/05 09:15:21 tom Exp $
  *
- *  formbox.c -- implements the form (i.e, some pairs label/editbox)
+ *  formbox.c -- implements the form (i.e., some pairs label/editbox)
  *
- *  Copyright 2003-2010,2011	Thomas E. Dickey
+ *  Copyright 2003-2018,2019	Thomas E. Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License, version 2.1
@@ -151,12 +151,13 @@ static int
 set_choice(DIALOG_FORMITEM item[], int choice, int item_no, bool * noneditable)
 {
     int result = -1;
-    int i;
 
     *noneditable = FALSE;
     if (!is_readonly(&item[choice])) {
 	result = choice;
     } else {
+	int i;
+
 	for (i = 0; i < item_no; i++) {
 	    if (!is_readonly(&(item[i]))) {
 		result = i;
@@ -287,45 +288,53 @@ tab_next(WINDOW *win,
 static bool
 scroll_next(WINDOW *win, DIALOG_FORMITEM item[], int stepsize, int *choice, int *scrollamt)
 {
+    bool result = TRUE;
     int old_choice = *choice;
     int old_scroll = *scrollamt;
     int old_row = MIN(item[old_choice].text_y, item[old_choice].name_y);
     int target = old_scroll + stepsize;
-    int n;
 
     if (stepsize < 0) {
 	if (old_row != old_scroll)
 	    target = old_scroll;
 	else
 	    target = old_scroll + stepsize;
-	if (target < 0)
-	    target = 0;
+	if (target < 0) {
+	    result = FALSE;
+	}
     } else {
-	int limit = form_limit(item);
-	if (target > limit)
-	    target = limit;
-    }
-
-    for (n = 0; item[n].name != 0; ++n) {
-	if (item[n].text_flen > 0) {
-	    int new_row = MIN(item[n].text_y, item[n].name_y);
-	    if (abs(new_row - target) < abs(old_row - target)) {
-		old_row = new_row;
-		*choice = n;
-	    }
+	if (target > form_limit(item)) {
+	    result = FALSE;
 	}
     }
 
-    if (old_choice != *choice)
-	print_item(win, item + old_choice, *scrollamt, FALSE);
+    if (result) {
+	int n;
 
-    *scrollamt = *choice;
-    if (*scrollamt != old_scroll) {
-	scrollok(win, TRUE);
-	wscrl(win, *scrollamt - old_scroll);
-	scrollok(win, FALSE);
+	for (n = 0; item[n].name != 0; ++n) {
+	    if (item[n].text_flen > 0) {
+		int new_row = MIN(item[n].text_y, item[n].name_y);
+		if (abs(new_row - target) < abs(old_row - target)) {
+		    old_row = new_row;
+		    *choice = n;
+		}
+	    }
+	}
+
+	if (old_choice != *choice)
+	    print_item(win, item + old_choice, *scrollamt, FALSE);
+
+	*scrollamt = *choice;
+	if (*scrollamt != old_scroll) {
+	    scrollok(win, TRUE);
+	    wscrl(win, *scrollamt - old_scroll);
+	    scrollok(win, FALSE);
+	}
+	result = (old_choice != *choice) || (old_scroll != *scrollamt);
     }
-    return (old_choice != *choice) || (old_scroll != *scrollamt);
+    if (!result)
+	beep();
+    return result;
 }
 
 /*
@@ -440,14 +449,16 @@ prev_valid_buttonindex(int state, int extra, bool non_editable)
 	DLG_KEYS_DATA( DLGK_FIELD_PREV, KEY_BTAB ), \
 	DLG_KEYS_DATA( DLGK_ITEM_NEXT,  CHR_NEXT ), \
 	DLG_KEYS_DATA( DLGK_ITEM_NEXT,  KEY_DOWN ), \
+	DLG_KEYS_DATA( DLGK_ITEM_NEXT,  KEY_RIGHT ), \
 	DLG_KEYS_DATA( DLGK_ITEM_NEXT,  KEY_NEXT ), \
 	DLG_KEYS_DATA( DLGK_ITEM_PREV,  CHR_PREVIOUS ), \
 	DLG_KEYS_DATA( DLGK_ITEM_PREV,  KEY_PREVIOUS ), \
+	DLG_KEYS_DATA( DLGK_ITEM_PREV,  KEY_LEFT ), \
 	DLG_KEYS_DATA( DLGK_ITEM_PREV,  KEY_UP ), \
 	DLG_KEYS_DATA( DLGK_PAGE_NEXT,  KEY_NPAGE ), \
 	DLG_KEYS_DATA( DLGK_PAGE_PREV,  KEY_PPAGE )
 /*
- * Display a form for fulfill a number of fields
+ * Display a form for entering a number of fields
  */
 int
 dlg_form(const char *title,
@@ -464,6 +475,7 @@ dlg_form(const char *title,
 	HELPKEY_BINDINGS,
 	ENTERKEY_BINDINGS,
 	NAVIGATE_BINDINGS,
+	TOGGLEKEY_BINDINGS,
 	END_KEYS_BINDING
     };
     static DLG_KEYS_BINDING binding2[] = {
@@ -471,6 +483,7 @@ dlg_form(const char *title,
 	HELPKEY_BINDINGS,
 	ENTERKEY_BINDINGS,
 	NAVIGATE_BINDINGS,
+	/* no TOGGLEKEY_BINDINGS, since that includes space... */
 	END_KEYS_BINDING
     };
     /* *INDENT-ON* */
@@ -481,10 +494,12 @@ dlg_form(const char *title,
 #endif
 
     int form_width;
-    int first = TRUE;
-    int first_trace = TRUE;
+    bool first = TRUE;
+    bool first_trace = TRUE;
     int chr_offset = 0;
-    int state = dialog_vars.defaultno ? dlg_defaultno_button() : sTEXT;
+    int state = (dialog_vars.default_button >= 0
+		 ? dlg_default_button()
+		 : sTEXT);
     int x, y, cur_x, cur_y, box_x, box_y;
     int code;
     int key = 0;
@@ -500,19 +515,32 @@ dlg_form(const char *title,
     bool field_changed = FALSE;
     bool non_editable = FALSE;
     WINDOW *dialog, *form;
-    char *prompt = dlg_strclone(cprompt);
+    char *prompt;
     const char **buttons = dlg_ok_labels();
     DIALOG_FORMITEM *current;
 
+    DLG_TRACE(("# %sform args:\n", (dialog_vars.formitem_type
+				    ? "password"
+				    : "")));
+    DLG_TRACE2S("title", title);
+    DLG_TRACE2S("message", cprompt);
+    DLG_TRACE2N("height", height);
+    DLG_TRACE2N("width", width);
+    DLG_TRACE2N("lheight", form_height);
+    DLG_TRACE2N("llength", item_no);
+    /* FIXME dump the items[][] too */
+    DLG_TRACE2N("current", *current_item);
+
     make_FORM_ELTs(items, item_no, &min_height, &min_width);
     dlg_button_layout(buttons, &min_width);
     dlg_does_output();
-    dlg_tab_correct_str(prompt);
 
 #ifdef KEY_RESIZE
   retry:
 #endif
 
+    prompt = dlg_strclone(cprompt);
+    dlg_tab_correct_str(prompt);
     dlg_auto_size(title, prompt, &height, &width,
 		  1 + 3 * MARGIN,
 		  MAX(26, 2 + min_width));
@@ -547,7 +575,7 @@ dlg_form(const char *title,
     dlg_draw_bottom_box2(dialog, border_attr, border2_attr, dialog_attr);
     dlg_draw_title(dialog, title);
 
-    (void) wattrset(dialog, dialog_attr);
+    dlg_attrset(dialog, dialog_attr);
     dlg_print_autowrap(dialog, prompt, height, width);
 
     form_width = width - 6;
@@ -626,12 +654,16 @@ dlg_form(const char *title,
 			    current->text_x,
 			    current->text_len,
 			    is_hidden(current), first);
+	    wsyncup(form);
+	    wcursyncup(form);
 	    field_changed = FALSE;
 	}
 
 	key = dlg_mouse_wgetch((state == sTEXT) ? form : dialog, &fkey);
-	if (dlg_result_key(key, fkey, &result))
-	    break;
+	if (dlg_result_key(key, fkey, &result)) {
+	    if (!dlg_button_key(result, &code, &key, &fkey))
+		break;
+	}
 
 	/* handle non-functionkeys */
 	if (!fkey) {
@@ -642,10 +674,6 @@ dlg_form(const char *title,
 		    result = dlg_ok_buttoncode(code);
 		    continue;
 		}
-		if (key == ' ') {
-		    fkey = TRUE;
-		    key = DLGK_ENTER;
-		}
 	    }
 	}
 
@@ -668,6 +696,7 @@ dlg_form(const char *title,
 		move_by = form_height;
 		break;
 
+	    case DLGK_TOGGLE:
 	    case DLGK_ENTER:
 		dlg_del_window(dialog);
 		result = (state >= 0) ? dlg_enter_buttoncode(state) : DLG_EXIT_OK;
@@ -750,14 +779,16 @@ dlg_form(const char *title,
 
 #ifdef KEY_RESIZE
 	    case KEY_RESIZE:
+		dlg_will_resize(dialog);
 		/* reset data */
 		height = old_height;
 		width = old_width;
-		/* repaint */
+		free(prompt);
 		dlg_clear();
+		dlg_unregister_window(form);
 		dlg_del_window(dialog);
-		refresh();
 		dlg_mouse_free_regions();
+		/* repaint */
 		goto retry;
 #endif
 	    default:
@@ -852,6 +883,7 @@ dlg_form(const char *title,
     }
 
     dlg_mouse_free_regions();
+    dlg_unregister_window(form);
     dlg_del_window(dialog);
     free(prompt);
 
@@ -901,11 +933,12 @@ dialog_form(const char *title,
 	    char **items)
 {
     int result;
-    int choice;
+    int choice = 0;
     int i;
     DIALOG_FORMITEM *listitems;
     DIALOG_VARS save_vars;
     bool show_status = FALSE;
+    char *help_result;
 
     dlg_save_vars(&save_vars);
     dialog_vars.separate_output = TRUE;
@@ -945,14 +978,9 @@ dialog_form(const char *title,
 	show_status = TRUE;
 	break;
     case DLG_EXIT_HELP:
-	dlg_add_result("HELP ");
+	dlg_add_help_formitem(&result, &help_result, &listitems[choice]);
 	show_status = dialog_vars.help_status;
-	if (USE_ITEM_HELP(listitems[choice].help)) {
-	    dlg_add_string(listitems[choice].help);
-	    result = DLG_EXIT_ITEM_HELP;
-	} else {
-	    dlg_add_string(listitems[choice].name);
-	}
+	dlg_add_string(help_result);
 	if (show_status)
 	    dlg_add_separator();
 	break;
@@ -964,6 +992,7 @@ dialog_form(const char *title,
 		dlg_add_separator();
 	    }
 	}
+	dlg_add_last_key(-1);
     }
 
     dlg_free_formitems(listitems);
