@@ -1,9 +1,9 @@
 /*
- *  $Id: inputbox.c,v 1.78 2016/08/28 13:55:10 tom Exp $
+ *  $Id: inputbox.c,v 1.87 2019/08/08 21:28:56 tom Exp $
  *
  *  inputbox.c -- implements the input box
  *
- *  Copyright 2000-2012,2016 Thomas E. Dickey
+ *  Copyright 2000-2018,2019 Thomas E. Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License, version 2.1
@@ -24,7 +24,7 @@
  *	Savio Lam (lam836@cs.cuhk.hk)
  */
 
-#include <dialog.h>
+#include <dlg_internals.h>
 #include <dlg_keys.h>
 
 #define sTEXT -1
@@ -74,8 +74,8 @@ dialog_inputbox(const char *title, const char *cprompt, int height, int width,
     int key, fkey, code;
     int result = DLG_EXIT_UNKNOWN;
     int state;
-    int first;
-    int edited;
+    bool first;
+    bool edited;
     char *input;
     WINDOW *dialog;
     WINDOW *editor;
@@ -84,6 +84,14 @@ dialog_inputbox(const char *title, const char *cprompt, int height, int width,
 
     dlg_does_output();
 
+    DLG_TRACE(("# inputbox args:\n"));
+    DLG_TRACE2S("title", title);
+    DLG_TRACE2S("message", cprompt);
+    DLG_TRACE2N("height", height);
+    DLG_TRACE2N("width", width);
+    DLG_TRACE2S("init", init);
+    DLG_TRACE2N("password", password);
+
     dlg_tab_correct_str(prompt);
 
     /* Set up the initial value */
@@ -124,7 +132,7 @@ dialog_inputbox(const char *title, const char *cprompt, int height, int width,
     dlg_draw_bottom_box2(dialog, border_attr, border2_attr, dialog_attr);
     dlg_draw_title(dialog, title);
 
-    (void) wattrset(dialog, dialog_attr);
+    dlg_attrset(dialog, dialog_attr);
     dlg_draw_helpline(dialog, FALSE);
     dlg_print_autowrap(dialog, prompt, height, width);
 
@@ -144,7 +152,7 @@ dialog_inputbox(const char *title, const char *cprompt, int height, int width,
 
     if (*input != '\0') {
 	dlg_show_string(editor, input, chr_offset, inputbox_attr,
-			0, 0, box_width, password, first);
+			0, 0, box_width, (bool) (password != 0), first);
 	wsyncup(editor);
 	wcursyncup(editor);
     }
@@ -164,14 +172,16 @@ dialog_inputbox(const char *title, const char *cprompt, int height, int width,
 	if (!first) {
 	    if (*input != '\0' && !edited) {
 		dlg_show_string(editor, input, chr_offset, inputbox_attr,
-				0, 0, box_width, password, first);
+				0, 0, box_width, (bool) (password != 0), first);
 		wmove(editor, 0, chr_offset);
 		wsyncup(editor);
 		wcursyncup(editor);
 	    }
 	    key = dlg_mouse_wgetch((state == sTEXT) ? editor : dialog, &fkey);
-	    if (dlg_result_key(key, fkey, &result))
-		break;
+	    if (dlg_result_key(key, fkey, &result)) {
+		if (!dlg_button_key(result, &code, &key, &fkey))
+		    break;
+	    }
 	}
 
 	/*
@@ -190,7 +200,7 @@ dialog_inputbox(const char *title, const char *cprompt, int height, int width,
 
 	    if (edit) {
 		dlg_show_string(editor, input, chr_offset, inputbox_attr,
-				0, 0, box_width, password, first);
+				0, 0, box_width, (bool) (password != 0), first);
 		wsyncup(editor);
 		wcursyncup(editor);
 		first = FALSE;
@@ -230,6 +240,7 @@ dialog_inputbox(const char *title, const char *cprompt, int height, int width,
 		break;
 #ifdef KEY_RESIZE
 	    case KEY_RESIZE:
+		dlg_will_resize(dialog);
 		/* reset data */
 		height = old_height;
 		width = old_width;
@@ -244,10 +255,11 @@ dialog_inputbox(const char *title, const char *cprompt, int height, int width,
 		beep();
 		break;
 	    }
-	} else {
+	} else if (key > 0) {
 	    beep();
 	}
     }
+    AddLastKey();
 
     dlg_unregister_window(editor);
     dlg_del_window(dialog);
