@@ -1,9 +1,9 @@
 /*
- *  $Id: menubox.c,v 1.159 2018/06/21 23:28:56 tom Exp $
+ *  $Id: menubox.c,v 1.164 2019/08/08 21:00:23 tom Exp $
  *
  *  menubox.c -- implements the menu box
  *
- *  Copyright 2000-2016,2018	Thomas E. Dickey
+ *  Copyright 2000-2018,2019	Thomas E. Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public Licens, version 2.1e
@@ -24,7 +24,7 @@
  *	Savio Lam (lam836@cs.cuhk.hk)
  */
 
-#include <dialog.h>
+#include <dlg_internals.h>
 #include <dlg_keys.h>
 
 typedef enum {
@@ -68,7 +68,6 @@ print_item(ALL_DATA * data,
 	   bool is_inputmenu)
 {
     chtype save = dlg_get_attrs(win);
-    int n;
     int climit = (data->item_x - data->tag_x - GUTTER);
     int my_width = data->menu_width;
     int my_x = data->item_x;
@@ -95,6 +94,8 @@ print_item(ALL_DATA * data,
 
     /* Clear 'residue' of last item and mark current current item */
     if (is_inputmenu) {
+	int n;
+
 	dlg_attrset(win, (selected != Unselected) ? item_selected_attr : item_attr);
 	for (n = my_y - 1; n < my_y + INPUT_ROWS - 1; n++) {
 	    wmove(win, n, 0);
@@ -203,6 +204,7 @@ handle_button(int code, DIALOG_LISTITEM * items, int choice)
 	dlg_add_string(help_result);
 	break;
     }
+    AddLastKey();
     return code;
 }
 
@@ -215,6 +217,7 @@ dlg_renamed_menutext(DIALOG_LISTITEM * items, int current, char *newtext)
     dlg_add_string(items[current].name);
     dlg_add_result(" ");
     dlg_add_string(newtext);
+    AddLastKey();
     return DLG_EXIT_EXTRA;
 }
 
@@ -517,8 +520,10 @@ dlg_menu(const char *title,
 		  all.box_x + all.tag_x + 1);
 
 	key = dlg_mouse_wgetch(dialog, &fkey);
-	if (dlg_result_key(key, fkey, &result))
-	    break;
+	if (dlg_result_key(key, fkey, &result)) {
+	    if (!dlg_button_key(result, &button, &key, &fkey))
+		break;
+	}
 
 	found = FALSE;
 	if (fkey) {
@@ -723,10 +728,6 @@ dlg_menu(const char *title,
 	    case KEY_RESIZE:
 		dlg_will_resize(dialog);
 		/* reset data */
-#define resizeit(name, NAME) \
-		name = ((NAME >= old_##NAME) \
-			? (NAME - (old_##NAME - old_##name)) \
-			: old_##name)
 		resizeit(height, LINES);
 		resizeit(width, COLS);
 		free(prompt);
