@@ -1,9 +1,9 @@
 /*
- *  $Id: pause.c,v 1.33 2011/10/20 23:35:17 tom Exp $
+ *  $Id: pause.c,v 1.41 2019/08/07 00:36:01 tom Exp $
  *
  *  pause.c -- implements the pause dialog
  *
- *  Copyright 2004-2010,2011	Thomas E. Dickey
+ *  Copyright 2004-2018,2019	Thomas E. Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License, version 2.1
@@ -55,13 +55,7 @@ dialog_pause(const char *title,
     static DLG_KEYS_BINDING binding[] = {
 	HELPKEY_BINDINGS,
 	ENTERKEY_BINDINGS,
-	DLG_KEYS_DATA( DLGK_ENTER,	' ' ),
-	DLG_KEYS_DATA( DLGK_FIELD_NEXT,	KEY_DOWN ),
-	DLG_KEYS_DATA( DLGK_FIELD_NEXT, KEY_RIGHT ),
-	DLG_KEYS_DATA( DLGK_FIELD_NEXT, TAB ),
-	DLG_KEYS_DATA( DLGK_FIELD_PREV,	KEY_UP ),
-	DLG_KEYS_DATA( DLGK_FIELD_PREV, KEY_BTAB ),
-	DLG_KEYS_DATA( DLGK_FIELD_PREV, KEY_LEFT ),
+	TRAVERSE_BINDINGS,
 	END_KEYS_BINDING
     };
     /* *INDENT-ON* */
@@ -72,7 +66,7 @@ dialog_pause(const char *title,
 #endif
 
     int i, x, y, step;
-    int button = 0;
+    int button = dlg_default_button();
     int seconds_orig;
     WINDOW *dialog;
     const char **buttons = dlg_ok_labels();
@@ -82,20 +76,28 @@ dialog_pause(const char *title,
     int result = DLG_EXIT_UNKNOWN;
     int button_high = (have_buttons ? BTN_HIGH : MARGIN);
     int gauge_y;
-    char *prompt = dlg_strclone(cprompt);
+    char *prompt;
+    int save_timeout = dialog_vars.timeout_secs;
+
+    DLG_TRACE(("# pause args:\n"));
+    DLG_TRACE2S("title", title);
+    DLG_TRACE2S("message", cprompt);
+    DLG_TRACE2N("height", height);
+    DLG_TRACE2N("width", width);
+    DLG_TRACE2N("seconds", seconds);
 
     curs_set(0);
 
-    dlg_tab_correct_str(prompt);
-
+    dialog_vars.timeout_secs = 0;
     seconds_orig = (seconds > 0) ? seconds : 1;
 
 #ifdef KEY_RESIZE
   retry:
-    height = old_height;
-    width = old_width;
 #endif
 
+    prompt = dlg_strclone(cprompt);
+    dlg_tab_correct_str(prompt);
+
     if (have_buttons) {
 	dlg_auto_size(title, prompt, &height, &width,
 		      MIN_HIGH,
@@ -129,7 +131,7 @@ dialog_pause(const char *title,
 	dlg_draw_title(dialog, title);
 	dlg_draw_helpline(dialog, FALSE);
 
-	(void) wattrset(dialog, dialog_attr);
+	dlg_attrset(dialog, dialog_attr);
 	dlg_print_autowrap(dialog, prompt, height, width);
 
 	dlg_draw_box2(dialog,
@@ -145,7 +147,7 @@ dialog_pause(const char *title,
 	 * attribute.
 	 */
 	(void) wmove(dialog, gauge_y + MARGIN, 4);
-	(void) wattrset(dialog, title_attr);
+	dlg_attrset(dialog, title_attr);
 
 	for (i = 0; i < (width - 2 * (3 + MARGIN)); i++)
 	    (void) waddch(dialog, ' ');
@@ -160,9 +162,9 @@ dialog_pause(const char *title,
 	 */
 	x = (seconds * (width - 2 * (3 + MARGIN))) / seconds_orig;
 	if ((title_attr & A_REVERSE) != 0) {
-	    wattroff(dialog, A_REVERSE);
+	    dlg_attroff(dialog, A_REVERSE);
 	} else {
-	    (void) wattrset(dialog, A_REVERSE);
+	    dlg_attrset(dialog, A_REVERSE);
 	}
 	(void) wmove(dialog, gauge_y + MARGIN, 4);
 	for (i = 0; i < x; i++) {
@@ -192,16 +194,20 @@ dialog_pause(const char *title,
 	    key = dlg_mouse_wgetch_nowait(dialog, &fkey);
 	    if (key == ERR) {
 		;		/* ignore errors in nodelay mode */
-	    } else {
-		if (dlg_result_key(key, fkey, &result))
+	    } else if (dlg_result_key(key, fkey, &result)) {
+		if (!dlg_button_key(result, &button, &key, &fkey))
 		    break;
 	    }
 
 	    switch (key) {
 #ifdef KEY_RESIZE
 	    case KEY_RESIZE:
+		dlg_will_resize(dialog);
 		dlg_clear();	/* fill the background */
 		dlg_del_window(dialog);		/* delete this window */
+		height = old_height;
+		width = old_width;
+		free(prompt);
 		refresh();	/* get it all onto the terminal */
 		goto retry;
 #endif
@@ -226,24 +232,26 @@ dialog_pause(const char *title,
 	    case DLGK_ENTER:
 		result = dlg_enter_buttoncode(button);
 		break;
-	    case DLGK_MOUSE(0):
-		result = DLG_EXIT_OK;
-		break;
-	    case DLGK_MOUSE(1):
-		result = DLG_EXIT_CANCEL;
-		break;
 	    case ERR:
 		break;
 	    default:
+		if (is_DLGK_MOUSE(key)) {
+		    result = dlg_ok_buttoncode(key - M_EVENT);
+		    if (result < 0)
+			result = DLG_EXIT_OK;
+		}
 		break;
 	    }
 	}
     } while ((result == DLG_EXIT_UNKNOWN) && (seconds-- > 0));
+    dlg_add_last_key(-1);
 
-    nodelay(dialog, FALSE);
     curs_set(1);
     dlg_mouse_free_regions();
     dlg_del_window(dialog);
     free(prompt);
+
+    dialog_vars.timeout_secs = save_timeout;
+
     return ((result == DLG_EXIT_UNKNOWN) ? DLG_EXIT_OK : result);
 }
