@@ -1,9 +1,9 @@
 /*
- *  $Id: rangebox.c,v 1.21 2017/01/31 00:27:21 tom Exp $
+ *  $Id: rangebox.c,v 1.27 2019/08/08 21:28:56 tom Exp $
  *
  *  rangebox.c -- implements the rangebox dialog
  *
- *  Copyright 2012-2016,2017	Thomas E. Dickey
+ *  Copyright 2012-2018,2019	Thomas E. Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License, version 2.1
@@ -21,7 +21,7 @@
  *	Boston, MA 02110, USA.
  */
 
-#include <dialog.h>
+#include <dlg_internals.h>
 #include <dlg_keys.h>
 
 #define ONE_HIGH 1
@@ -29,19 +29,6 @@
 #define MIN_HIGH (ONE_HIGH + 1 + (4 * MARGIN))
 #define MIN_WIDE (10 + 2 + (2 * MARGIN))
 
-struct _box;
-
-typedef struct _box {
-    WINDOW *parent;
-    WINDOW *window;
-    int x;
-    int y;
-    int width;
-    int height;
-    int period;
-    int value;
-} BOX;
-
 typedef struct {
     /* window in which the value and slider are drawn */
     WINDOW *window;
@@ -128,7 +115,7 @@ draw_value(VALUE * data, int value)
 	    scaled = offset;
 	}
 
-	(void) wattrset(win, gauge_attr);
+	dlg_attrset(win, gauge_attr);
 	wmove(win, data->slide_y, data->slide_x);
 	for (n = 0; n < data->slide_len; ++n) {
 	    (void) waddch(win, ' ');
@@ -136,9 +123,9 @@ draw_value(VALUE * data, int value)
 	wmove(win, data->slide_y, data->value_x);
 	wprintw(win, "%*d", data->value_len, value);
 	if ((gauge_attr & A_REVERSE) != 0) {
-	    wattroff(win, A_REVERSE);
+	    dlg_attroff(win, A_REVERSE);
 	} else {
-	    (void) wattrset(win, A_REVERSE);
+	    dlg_attrset(win, A_REVERSE);
 	}
 	wmove(win, data->slide_y, data->slide_x);
 	for (n = 0; n < scaled; ++n) {
@@ -148,7 +135,7 @@ draw_value(VALUE * data, int value)
 	    }
 	    (void) waddch(win, ch2);
 	}
-	(void) wattrset(win, dialog_attr);
+	dlg_attrset(win, dialog_attr);
 
 	wmove(win, y, x);
 	data->current = value;
@@ -215,13 +202,22 @@ dialog_rangebox(const char *title,
     WINDOW *dialog;
     int state = dlg_default_button();
     const char **buttons = dlg_ok_labels();
-    char *prompt = dlg_strclone(cprompt);
+    char *prompt;
     char buffer[MAX_LEN];
     int cur_value = default_value;
     int usable;
     int ranges;
     int yorg, xorg;
 
+    DLG_TRACE(("# tailbox args:\n"));
+    DLG_TRACE2S("title", title);
+    DLG_TRACE2S("message", cprompt);
+    DLG_TRACE2N("height", height);
+    DLG_TRACE2N("width", width);
+    DLG_TRACE2N("minval", min_value);
+    DLG_TRACE2N("maxval", max_value);
+    DLG_TRACE2N("default", default_value);
+
     if (max_value < min_value)
 	max_value = min_value;
     if (cur_value > max_value)
@@ -235,7 +231,9 @@ dialog_rangebox(const char *title,
   retry:
 #endif
 
+    prompt = dlg_strclone(cprompt);
     dlg_auto_size(title, prompt, &height, &width, 0, 0);
+
     height += MIN_HIGH;
     if (width < MIN_WIDE)
 	width = MIN_WIDE;
@@ -298,7 +296,7 @@ dialog_rangebox(const char *title,
     dlg_draw_title(dialog, title);
     dlg_draw_helpline(dialog, FALSE);
 
-    (void) wattrset(dialog, dialog_attr);
+    dlg_attrset(dialog, dialog_attr);
     dlg_print_autowrap(dialog, prompt, height, width);
 
     dlg_trace_win(dialog);
@@ -312,8 +310,10 @@ dialog_rangebox(const char *title,
 	}
 
 	key = dlg_mouse_wgetch(dialog, &fkey);
-	if (dlg_result_key(key, fkey, &result))
-	    break;
+	if (dlg_result_key(key, fkey, &result)) {
+	    if (!dlg_button_key(result, &button, &key, &fkey))
+		break;
+	}
 
 	if ((key2 = dlg_char_to_button(key, buttons)) >= 0) {
 	    result = key2;
@@ -375,13 +375,14 @@ dialog_rangebox(const char *title,
 		    break;
 #ifdef KEY_RESIZE
 		case KEY_RESIZE:
+		    dlg_will_resize(dialog);
 		    /* reset data */
 		    height = old_height;
 		    width = old_width;
 		    /* repaint */
+		    free(prompt);
 		    dlg_clear();
 		    dlg_del_window(dialog);
-		    refresh();
 		    dlg_mouse_free_regions();
 		    goto retry;
 #endif
@@ -409,8 +410,7 @@ dialog_rangebox(const char *title,
 
     sprintf(buffer, "%d", cur_value);
     dlg_add_result(buffer);
-    dlg_add_separator();
-    dlg_add_last_key(-1);
+    AddLastKey();
 
     dlg_del_window(dialog);
     dlg_mouse_free_regions();
