@@ -1,9 +1,9 @@
 /*
- *  $Id: tailbox.c,v 1.63 2011/06/27 08:19:43 tom Exp $
+ *  $Id: tailbox.c,v 1.74 2019/07/24 23:46:36 tom Exp $
  *
  *  tailbox.c -- implements the tail box
  *
- *  Copyright 2000-2010,2011	Thomas E. Dickey
+ *  Copyright 2000-2018,2019	Thomas E. Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License, version 2.1
@@ -34,7 +34,7 @@ typedef struct {
     const char **buttons;
     int hscroll;
     int old_hscroll;
-    char line[MAX_LEN + 1];
+    char line[MAX_LEN + 2];
     off_t last_pos;
 } MY_OBJ;
 
@@ -88,14 +88,10 @@ print_line(MY_OBJ * obj, WINDOW *win, int row, int width)
 
     (void) wmove(win, row, 0);	/* move cursor to correct line */
     (void) waddch(win, ' ');
-#ifdef NCURSES_VERSION
     (void) waddnstr(win, line, MIN((int) strlen(line), width - 2));
-#else
-    line[MIN((int) strlen(line), width - 2)] = '\0';
-    waddstr(win, line);
-#endif
 
     getyx(win, y, x);
+    (void) y;
     /* Clear 'residue' of previous line */
     for (i = 0; i < width - x; i++)
 	(void) waddch(win, ' ');
@@ -108,12 +104,6 @@ static void
 last_lines(MY_OBJ * obj, int target)
 {
     FILE *fp = obj->obj.input;
-    size_t inx;
-    int count = 0;
-    char buf[BUFSIZ + 1];
-    size_t size_to_read;
-    size_t size_as_read;
-    long offset = 0;
     long fpos = 0;
 
     if (fseek(fp, 0L, SEEK_END) == -1
@@ -121,8 +111,16 @@ last_lines(MY_OBJ * obj, int target)
 	dlg_exiterr("Error moving file pointer in last_lines().");
 
     if (fpos != 0) {
+	long offset = 0;
+
 	++target;
 	for (;;) {
+	    size_t inx;
+	    size_t size_to_read;
+	    size_t size_as_read;
+	    int count = 0;
+	    char buf[BUFSIZ + 1];
+
 	    if (fpos >= BUFSIZ) {
 		size_to_read = BUFSIZ;
 	    } else {
@@ -218,12 +216,29 @@ handle_input(DIALOG_CALLBACK * cb)
     return TRUE;
 }
 
+static bool
+valid_callback(DIALOG_CALLBACK * cb)
+{
+    bool valid = FALSE;
+    DIALOG_CALLBACK *p;
+    for (p = dialog_state.getc_callbacks; p != 0; p = p->next) {
+	if (p == cb) {
+	    valid = TRUE;
+	    break;
+	}
+    }
+    return valid;
+}
+
 static bool
 handle_my_getc(DIALOG_CALLBACK * cb, int ch, int fkey, int *result)
 {
     MY_OBJ *obj = (MY_OBJ *) cb;
     bool done = FALSE;
 
+    if (!valid_callback(cb))
+	return FALSE;
+
     if (!fkey && dlg_char_to_button(ch, obj->buttons) == 0) {
 	ch = DLGK_ENTER;
 	fkey = TRUE;
@@ -280,7 +295,11 @@ handle_my_getc(DIALOG_CALLBACK * cb, int ch, int fkey, int *result)
  * Display text from a file in a dialog box, like in a "tail -f".
  */
 int
-dialog_tailbox(const char *title, const char *file, int height, int width, int bg_task)
+dialog_tailbox(const char *title,
+	       const char *filename,
+	       int height,
+	       int width,
+	       int bg_task)
 {
     /* *INDENT-OFF* */
     static DLG_KEYS_BINDING binding[] = {
@@ -310,14 +329,21 @@ dialog_tailbox(const char *title, const char *file, int height, int width, int b
     FILE *fd;
     int min_width = 12;
 
+    DLG_TRACE(("# tailbox args:\n"));
+    DLG_TRACE2S("title", title);
+    DLG_TRACE2S("filename", filename);
+    DLG_TRACE2N("height", height);
+    DLG_TRACE2N("width", width);
+    DLG_TRACE2N("bg_task", bg_task);
+
     /* Open input file for reading */
-    if ((fd = fopen(file, "rb")) == NULL)
+    if ((fd = fopen(filename, "rb")) == NULL)
 	dlg_exiterr("Can't open input file in dialog_tailbox().");
 
 #ifdef KEY_RESIZE
   retry:
 #endif
-    dlg_auto_sizefile(title, file, &height, &width, 2, min_width);
+    dlg_auto_sizefile(title, filename, &height, &width, 2, min_width);
     dlg_print_size(height, width);
     dlg_ctl_size(height, width);
 
@@ -336,8 +362,8 @@ dialog_tailbox(const char *title, const char *file, int height, int width, int b
 			  y + MARGIN,
 			  x + MARGIN);
 
-    dlg_draw_box(dialog, 0, 0, height, width, dialog_attr, border_attr);
-    dlg_draw_bottom_box(dialog);
+    dlg_draw_box2(dialog, 0, 0, height, width, dialog_attr, border_attr, border2_attr);
+    dlg_draw_bottom_box2(dialog, border_attr, border2_attr, dialog_attr);
     dlg_draw_title(dialog, title);
     dlg_draw_helpline(dialog, FALSE);
 
@@ -359,7 +385,7 @@ dialog_tailbox(const char *title, const char *file, int height, int width, int b
     obj->obj.handle_getc = handle_my_getc;
     obj->obj.handle_input = bg_task ? handle_input : 0;
     obj->obj.keep_bg = bg_task && dialog_vars.cant_kill;
-    obj->obj.bg_task = bg_task;
+    obj->obj.bg_task = (bool) bg_task;
     obj->text = text;
     obj->buttons = buttons;
     dlg_add_callback(&(obj->obj));
@@ -371,6 +397,7 @@ dialog_tailbox(const char *title, const char *file, int height, int width, int b
     dlg_attr_clear(text, thigh, getmaxx(text), dialog_attr);
     repaint_text(obj);
 
+    dlg_trace_win(dialog);
     if (bg_task) {
 	result = DLG_EXIT_OK;
     } else {
@@ -379,6 +406,7 @@ dialog_tailbox(const char *title, const char *file, int height, int width, int b
 	    ch = dlg_getc(dialog, &fkey);
 #ifdef KEY_RESIZE
 	    if (fkey && ch == KEY_RESIZE) {
+		dlg_will_resize(dialog);
 		/* reset data */
 		height = old_height;
 		width = old_width;
