@@ -1,9 +1,9 @@
 /*
- * $Id: timebox.c,v 1.56 2016/08/28 14:16:33 tom Exp $
+ * $Id: timebox.c,v 1.63 2019/08/08 21:28:56 tom Exp $
  *
  *  timebox.c -- implements the timebox dialog
  *
- *  Copyright 2001-2013,2016   Thomas E. Dickey
+ *  Copyright 2001-2018,2019   Thomas E. Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License, version 2.1
@@ -21,7 +21,7 @@
  *	Boston, MA 02110, USA.
  */
 
-#include <dialog.h>
+#include <dlg_internals.h>
 #include <dlg_keys.h>
 
 #include <time.h>
@@ -82,7 +82,7 @@ draw_cell(BOX * data)
 		 data->height + (2 * MARGIN), data->width + (2 * MARGIN),
 		 menubox_border_attr, menubox_border2_attr);
 
-    (void) wattrset(data->window, item_attr);
+    dlg_attrset(data->window, item_attr);
     wprintw(data->window, "%02d", data->value);
     return 0;
 }
@@ -180,14 +180,23 @@ dialog_timebox(const char *title,
     int button;
     int result = DLG_EXIT_UNKNOWN;
     WINDOW *dialog;
-    time_t now_time = time((time_t *) 0);
+    time_t now_time;
     struct tm current;
     int state = dlg_default_button();
     const char **buttons = dlg_ok_labels();
-    char *prompt = dlg_strclone(subtitle);
+    char *prompt;
     char buffer[MAX_LEN];
     DIALOG_VARS save_vars;
 
+    DLG_TRACE(("# timebox args:\n"));
+    DLG_TRACE2S("title", title);
+    DLG_TRACE2S("message", subtitle);
+    DLG_TRACE2N("height", height);
+    DLG_TRACE2N("width", width);
+    DLG_TRACE2N("hour", hour);
+    DLG_TRACE2N("minute", minute);
+    DLG_TRACE2N("second", second);
+
     now_time = time((time_t *) 0);
     current = *localtime(&now_time);
 
@@ -200,7 +209,9 @@ dialog_timebox(const char *title,
   retry:
 #endif
 
+    prompt = dlg_strclone(subtitle);
     dlg_auto_size(title, prompt, &height, &width, 0, 0);
+
     height += MIN_HIGH;
     if (width < MIN_WIDE)
 	width = MIN_WIDE;
@@ -224,7 +235,7 @@ dialog_timebox(const char *title,
     dlg_draw_title(dialog, title);
     dlg_draw_helpline(dialog, FALSE);
 
-    (void) wattrset(dialog, dialog_attr);
+    dlg_attrset(dialog, dialog_attr);
     dlg_print_autowrap(dialog, prompt, height, width);
 
     /* compute positions of hour, month and year boxes */
@@ -285,8 +296,10 @@ dialog_timebox(const char *title,
 	    dlg_set_focus(dialog, obj->window);
 
 	key = dlg_mouse_wgetch(dialog, &fkey);
-	if (dlg_result_key(key, fkey, &result))
-	    break;
+	if (dlg_result_key(key, fkey, &result)) {
+	    if (!dlg_button_key(result, &button, &key, &fkey))
+		break;
+	}
 
 	if ((key2 = dlg_char_to_button(key, buttons)) >= 0) {
 	    result = key2;
@@ -341,6 +354,7 @@ dialog_timebox(const char *title,
 		    break;
 #ifdef KEY_RESIZE
 		case KEY_RESIZE:
+		    dlg_will_resize(dialog);
 		    /* reset data */
 		    height = old_height;
 		    width = old_width;
@@ -348,9 +362,9 @@ dialog_timebox(const char *title,
 		    minute = mn_box.value;
 		    second = sc_box.value;
 		    /* repaint */
+		    free(prompt);
 		    dlg_clear();
 		    dlg_del_window(dialog);
-		    refresh();
 		    dlg_mouse_free_regions();
 		    goto retry;
 #endif
@@ -382,7 +396,7 @@ dialog_timebox(const char *title,
 			beep();
 		    }
 		}
-	    } else {
+	    } else if (key > 0) {
 		beep();
 	    }
 	}
@@ -412,8 +426,7 @@ dialog_timebox(const char *title,
 	DefaultFormat(buffer, hr_box);
 
     dlg_add_result(buffer);
-    dlg_add_separator();
-    dlg_add_last_key(-1);
+    AddLastKey();
 
     return CleanupResult(result, dialog, prompt, &save_vars);
 }
