@@ -1,9 +1,9 @@
 /*
- *  $Id: treeview.c,v 1.32 2018/06/19 22:57:01 tom Exp $
+ *  $Id: treeview.c,v 1.37 2019/08/08 23:47:23 tom Exp $
  *
  *  treeview.c -- implements the treeview dialog
  *
- *  Copyright 2012-2016,2018	Thomas E. Dickey
+ *  Copyright 2012-2018,2019	Thomas E. Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License, version 2.1
@@ -21,7 +21,7 @@
  *	Boston, MA 02110, USA.
  */
 
-#include <dialog.h>
+#include <dlg_internals.h>
 #include <dlg_keys.h>
 
 #define INDENT 3
@@ -362,8 +362,10 @@ dlg_treeview(const char *title,
 	    wmove(dialog, box_y + choice + 1, box_x + all.check_x + 2);
 
 	key = dlg_mouse_wgetch(dialog, &fkey);
-	if (dlg_result_key(key, fkey, &result))
-	    break;
+	if (dlg_result_key(key, fkey, &result)) {
+	    if (!dlg_button_key(result, &button, &key, &fkey))
+		break;
+	}
 
 	was_mouse = (fkey && is_DLGK_MOUSE(key));
 	if (was_mouse)
@@ -552,7 +554,7 @@ dlg_treeview(const char *title,
 		    beep();
 		}
 	    }
-	} else {
+	} else if (key > 0) {
 	    beep();
 	}
     }
@@ -649,12 +651,11 @@ dialog_treeview(const char *title,
     if (show_status) {
 	for (i = 0; i < item_no; i++) {
 	    if (listitems[i].state) {
+		if (dlg_need_separator())
+		    dlg_add_separator();
 		if (dialog_vars.separate_output) {
 		    dlg_add_string(listitems[i].name);
-		    dlg_add_separator();
 		} else {
-		    if (dlg_need_separator())
-			dlg_add_separator();
 		    if (flag == FLAG_CHECK)
 			dlg_add_quoted(listitems[i].name);
 		    else
@@ -662,7 +663,7 @@ dialog_treeview(const char *title,
 		}
 	    }
 	}
-	dlg_add_last_key(-1);
+	AddLastKey();
     }
 
     dlg_free_columns(&listitems[0].text, (int) sizeof(DIALOG_LISTITEM), item_no);
