@@ -1,9 +1,9 @@
 /*
- *  $Id: yesno.c,v 1.62 2018/06/19 22:57:01 tom Exp $
+ *  $Id: yesno.c,v 1.65 2019/08/05 23:45:02 tom Exp $
  *
  *  yesno.c -- implements the yes/no box
  *
- *  Copyright 1999-2012,2018	Thomas E. Dickey
+ *  Copyright 1999-2018,2019	Thomas E. Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License, version 2.1
@@ -110,8 +110,10 @@ dialog_yesno(const char *title, const char *cprompt, int height, int width)
 	    show = FALSE;
 	}
 	key = dlg_mouse_wgetch(dialog, &fkey);
-	if (dlg_result_key(key, fkey, &result))
-	    break;
+	if (dlg_result_key(key, fkey, &result)) {
+	    if (!dlg_button_key(result, &button, &key, &fkey))
+		break;
+	}
 	if ((code = dlg_char_to_button(key, buttons)) >= 0) {
 	    result = dlg_ok_buttoncode(code);
 	    break;
@@ -161,10 +163,11 @@ dialog_yesno(const char *title, const char *cprompt, int height, int width)
 		}
 		break;
 	    }
-	} else {
+	} else if (key > 0) {
 	    beep();
 	}
     }
+    dlg_add_last_key(-1);
 
     dlg_del_window(dialog);
     dlg_mouse_free_regions();
